#!/usr/bin/env python

"""pic tool base classes

Display pics so we can record rotate data

$Id: Pic.py,v 1.6 2005/08/11 03:35:30 sjg Exp $
"""

import getopt, os, sys
from Tkinter import *
import Image, ImageTk

def getConfig(config, key, default=None):
    """return value of key if present in config, else default"""
    if config.has_key(key):
        return config[key]
    return default

# this is how thumnail scales
def scale(old, size):
    x, y = old
    if x > size[0]: y = y * size[0] / x; x = size[0]
    if y > size[1]: x = x * size[1] / y; y = size[1]
    return (x, y)


class Pic(Frame):
    
    def __init__(self, parent=None, pics=None, conf={}):
        self.conf = conf
        Frame.__init__(self, parent)
        self.doWeb = getConfig(conf, 'doWeb', False)
        self.pics = pics
        self.rotated = {}
        self.thm = None
        self.index = None
        self.setupPics()
        self.setupGui()
        if self.index >= 0:
            self.selectPic(self.index)

    def setupPics(self):
        if self.pics:
            self.index = 0
        
    def selectPic(self, index):
        self.index = index
        self.setPic(self.pics[index])

    def setPic(self, pic):
        self.picFile = pic
        self.picName, ext = os.path.splitext(os.path.basename(pic))
        im = Image.open(pic)
        if self.rotated.has_key(pic):
            self.rotate = self.rotated[pic]
        else:
            self.rotate = 0
	self.changed = 0
        #print im.info['exif']
        self.setImage(im, self.rotate)

    def setImage(self, image, rotate=0):
        if rotate:
            image = image.rotate(-rotate)
        self.image = image
        if self.doWeb:
            wsize = scale(image.size, map(int, getConfig(self.conf, 'webXY', '800x600').split('x')))
            if wsize != image.size:
                web = image.resize(wsize)
            else:
                web = image
                self.web = ImageTk.PhotoImage(web)
        tsize = scale(image.size, map(int, getConfig(self.conf, 'thmXY', '200x200').split('x')))
        self.thm = ImageTk.PhotoImage(image.resize(tsize))
        self.label.config(image=self.thm)
        self.label.pack(expand=YES, fill=BOTH)
        self.master.title("%s (%d/%d)" % (self.picName, self.index + 1, len(self.pics)))
        
    def setupGui(self):
        self.label = Label(self)
        self.addWidgets()
        self.pack(expand=YES, fill=BOTH)

    def addWidgets(self): pass
    
class PicTool(Pic):

    rotate_fp = None
    deleted = {}
    increment = 1;

    def setupPics(self):
        Pic.setupPics(self)
        self.rotateFile = getConfig(self.conf, 'rotateFile')
        if self.rotateFile and self.rotateFile != 'no' and self.rotateFile != '-' and os.path.exists(self.rotateFile):
            lines = open(self.rotateFile, "r").readlines()
            for line in lines:
                p, r = line.split(':')
                self.rotated[p] = int(r)

    def addWidgets(self):
        """bind some commands
        a - annotate
        d - delete
        e - exif
        f - file
        n - next
        r - rotate right
        l - rotate left
        s - save
        q - quit
        """
        self.label.bind('<KeyPress>', self.onKeyPress)
        self.label.focus()

    def notYet(self):
        print >> sys.stderr, "Not yet implemented!"

    def onKeyPress(self, event):
        if event.char == 'a':
            self.doAnnotate()
        elif event.char == 'd':
            self.doDelete()
        elif event.char == 'e':
            self.doExif()
        elif event.char == 'f':
            self.doFile()
        elif event.char == 'n':
            self.doNext()
        elif event.char == 'p':
            self.doPrevious()
        elif event.char == 'r':
            self.doRotate(90)
        elif event.char == 'l':
            self.doRotate(-90)
        elif event.char == 's':
            self.doSave()
        elif event.char == 'q':
            self.doQuit()

    def doAnnotate(self):
        self.notYet()

    def doDelete(self):
        self.notYet()

    def doExif(self):
        self.notYet()
        exifData = self.image.info['exif']
        #print `exifData`

    def doFile(self):
        print self.picFile

    def nextPic(self, all=False):
        i = self.index + self.increment
        while ((self.increment > 0 and i < len(self.pics)) or
               self.increment < 0 and i >= 0):
            if all or not self.deleted.has_key(i):
                self.selectPic(i)
                return
            i += self.increment
        self.maybeQuit()
        
    def doPrevious(self, all=False):
        self.increment = -1;
        self.nextPic(all)

    def doNext(self, all=False):
        self.increment = 1;
        self.nextPic(all)

    def maybeQuit(self):
        self.doQuit()
        
    def doQuit(self):
        sys.exit(0)

    def doRotate(self, rotate):
        # we record 90 for clockwise rotation, but image uses -90
        self.setImage(self.image.rotate(-rotate))
        self.rotate += rotate
	if self.rotate:
	   self.changed = 1;
        
    def doSave(self):
        self.rotated[self.picFile] = self.rotate
        if self.rotate != 0 and self.changed != 0:
            if not self.rotate_fp:
                self.rotateFile = getConfig(self.conf, 'rotateFile')
                if not self.rotateFile or self.rotateFile == 'no':
                    return
                elif self.rotateFile == '-':
                    self.rotate_fp = sys.stdout
                else:
                    self.rotate_fp = open(self.rotateFile, 'a')
            print >> self.rotate_fp, "%s:%d" % (self.picFile, self.rotate)
        self.changed = 0

        
def main(klass=PicTool, conf={}):

    xy = getConfig(conf, 'thmXY', '600x600')
    file = getConfig(conf, 'rotateFile', '.rotate')

    try:
        opts, args = getopt.getopt(sys.argv[1:], "g:r:")
    except getopt.GetoptError:
        print >> sys.stderr, "Use the source Luke!"
        sys.exit(2)
    for o, a in opts:
        if o == '-g':
            xy = a
        elif o == '-r':
            file = a
    conf['thmXY'] = xy
    conf['rotateFile'] = file

    w = klass(conf=conf, pics=args)
    w.mainloop()
    
if __name__ == '__main__':
    main()
